@@ -11,7 +11,10 @@
 
 #include "eb.h"
 
+#include <unistd.h>
+#include <fcntl.h>
 #include <time.h>
+#include <signal.h>
 
 #define HTTP_MUST_AUTHENTICATE 401
 
@@ -19,6 +22,7 @@ CURL *curl_handle = NULL;
 char *serverData;
 int serverDataLen;
 static char errorText[CURL_ERROR_SIZE + 1];
+static char *httpLanguage;
 
 static void curl_setError(CURLcode curlret, const char *url);
 static bool ftpConnect(const char *url);
@@ -311,7 +315,7 @@ parseRefresh(char *ref, int *delay_p)
 	    ++u;
 	else
 	    qc = 0;
-	strcpy(ref, u);
+	strmove(ref, u);
 	u = ref + strlen(ref);
 	if(u > ref && u[-1] == qc)
 	    u[-1] = 0;
@@ -389,7 +393,7 @@ copy_and_sanitize(const char *start, const char *end)
 	if(frag) {
 	    params = strchr(new_copy, '?');
 	    if(params && params > frag)
-		strcpy(frag, params);
+		strmove(frag, params);
 	    else
 		*frag = 0;
 	}
@@ -397,7 +401,7 @@ copy_and_sanitize(const char *start, const char *end)
 	getPortLocURL(new_copy, &portloc, 0);
 	if(portloc && !isdigit(portloc[1])) {
 	    const char *s = portloc + strcspn(portloc, "/?#\1");
-	    strcpy((char *)portloc, s);
+	    strmove((char *)portloc, s);
 	}
     }
 
@@ -451,7 +455,10 @@ httpConnect(const char *from, const char *url)
     } else if(mt = findMimeByProtocol(prot)) {
       mimeProcess:
 	cmd = pluginCommand(mt, url, 0);
+/* Stop ignoring SIGPIPE for the duration of system(): */
+	signal(SIGPIPE, SIG_DFL);
 	system(cmd);
+	signal(SIGPIPE, SIG_IGN);
 	nzFree(cmd);
 	return true;
     } else {
@@ -497,6 +504,12 @@ httpConnect(const char *from, const char *url)
     if(tmp_headers == NULL)
 	i_printfExit(MSG_NoMem);
     custom_headers = tmp_headers;
+    if(httpLanguage) {
+	custom_headers = curl_slist_append(custom_headers, httpLanguage);
+	/* Check for failure to allocate. */
+	if(custom_headers == NULL)
+	    i_printfExit(MSG_NoMem);
+    }
 
     post = strchr(url, '\1');
     postb = 0;
@@ -1094,6 +1107,35 @@ allIPs(void)
 	nzFree(domlist[k]);
 }				/* allIPs */
 
+/* If the user has asked for locale-specific responses, then build an
+ * appropriate Accept-Language: header. */
+void
+setHTTPLanguage(const char *lang)
+{
+    int httpLanguage_l;
+
+    httpLanguage = initString(&httpLanguage_l);
+    stringAndString(&httpLanguage, &httpLanguage_l, "Accept-Language: ");
+    stringAndString(&httpLanguage, &httpLanguage_l, lang);
+}				/* setHTTPLanguage */
+
+/* Set the FD_CLOEXEC flag on a socket newly-created by libcurl.
+ * Let's not leak libcurl's sockets to child processes created by the
+ * ! (escape-to-shell) command.
+ * This is a callback.  It returns 0 on success, 1 on failure, per the
+ * libcurl docs.
+ */
+static int
+my_curl_safeSocket(void *clientp, curl_socket_t socketfd, curlsocktype purpose)
+{
+    int success = fcntl(socketfd, F_SETFD, FD_CLOEXEC);
+    if(success == -1)
+	success = 1;
+    else
+	success = 0;
+    return success;
+}
+
 /* Clean up libcurl's state. */
 static void
 my_curl_cleanup(void)
@@ -1122,6 +1164,8 @@ my_curl_init(void)
 	i_printfExit(MSG_CurlVersion, major, minor, patch);
 
 /* Lots of these setopt calls shouldn't fail.  They just diddle a struct. */
+    curl_easy_setopt(curl_handle, CURLOPT_SOCKOPTFUNCTION, my_curl_safeSocket);
+
     curl_easy_setopt(curl_handle, CURLOPT_WRITEFUNCTION, eb_curl_callback);
     curl_easy_setopt(curl_handle, CURLOPT_HEADERFUNCTION, curl_header_callback);
     if(debugLevel >= 4)
@@ -1154,6 +1198,9 @@ my_curl_init(void)
 	goto libcurl_init_fail;
     curl_init_status =
        curl_easy_setopt(curl_handle, CURLOPT_COOKIEJAR, cookieFile);
+    if(curl_init_status != CURLE_OK)
+	goto libcurl_init_fail;
+    curl_init_status = curl_easy_setopt(curl_handle, CURLOPT_ENCODING, "");
     if(curl_init_status != CURLE_OK)
 	goto libcurl_init_fail;
     if(proxy_host)
